#include "BSL_Commands_P4xx.h"
#include "BSL_Commands_5xx.h"
#include "BSL_Command_Definitions_P4xx.h"
#include "BSL_Command_Definitions_5xx.h"
#include "BSL_IO_UART.h"
#include <stdio.h>

dataBuffer TXBuffer;
dataBuffer RXBuffer;
unsigned int answer;
unsigned char(*BSL_RX_Packet)(dataBuffer *db);
unsigned char(*BSL_TX_Packet)(dataBuffer  db);

/*******************************************************************************
*Function:    Flash_get_RX_Buffer_32
*Description: returns RX_Buffer
*Parameters:
*             none
*Returns:
*             dataBuffer RXBuffer
*******************************************************************************/
dataBuffer Flash_get_RX_Buffer_32()
{
	return RXBuffer;
}

/*******************************************************************************
*Function:    Flash_set_TX_Packet_32
*Description: sets BSL_TX_PACKET to rxFunc
*Parameters:
*             unsigned char (*txFunc)(dataBuffer db)	location of databyte
*Returns:
*             none
*******************************************************************************/
void Flash_set_TX_Packet_32(unsigned char(*txFunc)(dataBuffer db))
{
	BSL_TX_Packet = txFunc;
}

/*******************************************************************************
*Function:    Flash_set_RX_Packet_32
*Description: sets BSL_RX_Packet to rxFunc
*Parameters:
*             unsigned char (*rxFunc)(dataBuffer *db)	location of databyte
*Returns:
*             none
*******************************************************************************/
void Flash_set_RX_Packet_32(unsigned char(*rxFunc)(dataBuffer *db))
{
	BSL_RX_Packet = rxFunc;
}

/*******************************************************************************
*Function:    Flash_RX_DataBlock_32
*Description: Programs a block of data to the MSP432 memory
*Parameters:
*             DataBlock data        A struct including start address and data
*Returns:
*             unsigned char         A FLASH_ACK if the data was programmed correctly
*******************************************************************************/
unsigned char Flash_RX_DataBlock_32(DataBlock data)
{
	unsigned int i;

#ifdef DEBUG_VERBOSE
	printf("\n-------------------------------------\n");
	printf("Writing Data Block...\n");
#endif
	TXBuffer.data[0] = RX_DATA_BLOCK_32;
	TXBuffer.data[1] = (unsigned char)(data.startAddr & 0xFF);
	TXBuffer.data[2] = (unsigned char)(data.startAddr >> 8 & 0xFF);
	TXBuffer.data[3] = (unsigned char)(data.startAddr >> 16 & 0xFF);
	TXBuffer.data[4] = (unsigned char)(data.startAddr >> 24 & 0xFF);
	for (i = 0; i < data.numberOfBytes; i++)
	{
		TXBuffer.data[i + 5] = data.data[i];
	}
	TXBuffer.size = data.numberOfBytes + 5;
	return getBSL_Response(BSL_TX_Packet(TXBuffer));
}

/*******************************************************************************
*Function:    Flash_RX_Password_32
*Description: Sends a 32 byte password
*Parameters:
*             DataBlock data        A struct including a 32 byte 'password' array
*Returns:
*             unsigned char         An FLASH_ACK if the device was correctly unlocked
*Note:
*             An incorrect password will cause flash+RAM erase, so this function
*             could return a timeout as the MSP430 goes unresponsive
*******************************************************************************/
unsigned char Flash_RX_Password_32(DataBlock data)
{
	int i;
	TXBuffer.data[0] = RX_PASSWORD_32;
	for (i = 0; i <= 256; i++)
	{
		TXBuffer.data[i + 1] = data.data[i];
	}
	TXBuffer.size = 257;
	return getBSL_Response(BSL_TX_Packet(TXBuffer));
}

/*******************************************************************************
*Function:    Flash_eraseSegment_32
*Description: Erases a segment of device flash
*Parameters:
*             unsigned long int addr  The address within a flash segment to erase
*Returns:
*             unsigned char           An FLASH_ACK if the flash was correctly erased
*******************************************************************************/
unsigned char Flash_eraseSegment_32(unsigned long int addr)
{
	TXBuffer.data[0] = ERASE_SECTOR_32;
	TXBuffer.data[1] = (unsigned char)(addr & 0xFF);
	TXBuffer.data[2] = (unsigned char)(addr >> 8 & 0xFF);
	TXBuffer.data[3] = (unsigned char)(addr >> 16 & 0xFF);
	TXBuffer.data[4] = (unsigned char)(addr >> 24 & 0xFF);
	TXBuffer.size = 5;
	return getBSL_Response(BSL_TX_Packet(TXBuffer));
}

/*******************************************************************************
*Function:    Flash_rebootReset
*Description: Triggers a reboot reset
*Parameters:
*             none
*Returns:
*             none
*******************************************************************************/
unsigned char Flash_rebootReset()
{
	TXBuffer.data[0] = REBOOT_RESET;
	return FLASH_ACK;
}

/*******************************************************************************
*Function:    Flash_CRC_Check_32
*Description: Calculates a CRC check on the device
*Parameters:
*             unsigned long int addr    The address for the first byte included in the CRC
*             unsigned long int length  The number of bytes in the CRC
*Returns:
*             unsigned char             An FLASH_ACK if the dataBuffer is filled correctly
*DataBuffer:
*             [1]                   low bits of 16 bit crc
*             [2]                   high bits of 16 bit crc
*******************************************************************************/
unsigned char Flash_CRC_Check_32(unsigned long int addr, unsigned long int length)
{
	TXBuffer.data[0] = CRC_CHECK_32;
	TXBuffer.data[1] = (unsigned char)(addr & 0xFF);
	TXBuffer.data[2] = (unsigned char)(addr >> 8 & 0xFF);
	TXBuffer.data[3] = (unsigned char)(addr >> 16 & 0xFF);
	TXBuffer.data[4] = (unsigned char)(addr >> 24 & 0xFF);
	TXBuffer.data[5] = (unsigned char)(length & 0xFF);
	TXBuffer.data[6] = (unsigned char)(length >> 8 & 0xFF);
	TXBuffer.size = 7;
	answer = BSL_TX_Packet(TXBuffer);
	if (answer != FLASH_ACK)
	{
		return answer;
	}
	else
	{
		answer = BSL_RX_Packet(&RXBuffer);
		if (answer != FLASH_ACK)
		{
			return answer;
		}
		else if (RXBuffer.data[0] != BSL_DATA_REPLY)
		{
			return RXBuffer.data[1];
		}
		else
		{
			return FLASH_ACK;
		}
	}
}

/*******************************************************************************
*Function:    Flash_LoadPC_32
*Description: Sends a command to set the MSP430's Program counter to a location
*Parameters:
*             unsigned long int addr  The address to which the PC should be set
*Returns:
*             unsigned char           FLASH_ACK: the MSP430 received the command
*                                     the device can not be counted on being responsive after
*******************************************************************************/
unsigned char Flash_LoadPC_32(unsigned long int addr)
{
	TXBuffer.data[0] = LOAD_PC_32;
	TXBuffer.data[1] = (unsigned char)(addr & 0xFF);
	TXBuffer.data[2] = (unsigned char)(addr >> 8 & 0xFF);
	TXBuffer.data[3] = (unsigned char)(addr >> 16 & 0xFF);
	TXBuffer.data[4] = (unsigned char)(addr >> 24 & 0xFF);
	TXBuffer.size = 5;
	return BSL_TX_Packet(TXBuffer);
}

/*******************************************************************************
*Function:    Flash_TX_DataBlock_32
*Description: Transmits a block of data from the BSL using the dataBuffer
*Parameters:
*             unsigned long int addr     The address where the data should begin
*             unsigned long int length   The length of data in the data block
*             unsigned char *   buffer   A buffer to store the recieved Data from BSL
*Returns:
*             unsigned char              An FLASH_ACK if the dataBuffer is filled correctly
*******************************************************************************/
unsigned char Flash_TX_DataBlock_32(unsigned long int addr, unsigned long int length, unsigned char *buffer)
{
#ifdef DEBUG_VERBOSE
	printf("\n-------------------------------------\n");
	printf("Requesting Data Block...\n");
#endif
	TXBuffer.data[0] = TX_DATA_BLOCK_32;
	TXBuffer.data[1] = (unsigned char)(addr & 0xFF);
	TXBuffer.data[2] = (unsigned char)(addr >> 8 & 0xFF);
	TXBuffer.data[3] = (unsigned char)(addr >> 16 & 0xFF);
	TXBuffer.data[4] = (unsigned char)(addr >> 24 & 0xFF);
	TXBuffer.data[5] = (unsigned char)(length & 0xFF);
	TXBuffer.data[6] = (unsigned char)(length >> 8 & 0xFF);
	TXBuffer.size = 7;
	answer = BSL_TX_Packet(TXBuffer);
	if (answer != FLASH_ACK)
	{
		return answer;
	}
	else
	{
		unsigned int buffer_pointer = 0;
		for (buffer_pointer = 0; buffer_pointer < length;)
		{
			answer = BSL_RX_Packet(&RXBuffer);
			if (answer != FLASH_ACK)
			{
				return answer;
			}
			else if (RXBuffer.data[0] == BSL_DATA_REPLY)
			{
				unsigned int i = 0;
				for (i = 1; i < RXBuffer.size; i++, buffer_pointer++)
				{
					buffer[buffer_pointer] = RXBuffer.data[i];
				}
			}
			else
			{
				return FLASH_NACK;
			}
		}
		return 0;
	}
}

/*******************************************************************************
*Function:    Flash_TX_BSL_Version_32
*Description: Transmits the BSL version command and fills data buffer with reply
*Parameters:
*             none
*Returns:
*             unsigned char         An FLASH_ACK if the dataBuffer is filled correctly
*DataBuffer:
*             [1]                   Vendor Field
*             [2]                   Command Interpreter Version Field
*             [3]                   API Version Field
*             [4]                   Peripheral Interface Version Field
*             [5]...................Build ID
*******************************************************************************/
unsigned char Flash_TX_BSL_Version_32()
{
	TXBuffer.data[0] = TX_BSL_VERSION_32;
	TXBuffer.size = 1;
	answer = BSL_TX_Packet(TXBuffer);
	if (answer != FLASH_ACK)
	{
		return answer;
	}
	else
	{
		answer = BSL_RX_Packet(&RXBuffer);
		if (answer != FLASH_ACK)
		{
			return answer;
		}
		else if (RXBuffer.data[0] != BSL_DATA_REPLY)
		{
			return RXBuffer.data[1];
		}
		else
		{
			return FLASH_ACK;
		}
	}
}