/*==========================================================================*\
|                                                                            |
|                                                                            |
| PC-side Bootstrap Loader communication Application                         |
|                                                                            |
| See main.c for full version and legal information                          |
|                                                                            |
\*==========================================================================*/
#include "BSL_IO_UART.h"

void SetRSTpin(BOOL level);
void SetTESTpin(BOOL level);

HANDLE       hComPort;    /* COM-Port Handle             */
DCB          comDCB;      /* COM-Port Control-Settings   */
COMSTAT      comState;    /* COM-Port Status-Information */
COMMTIMEOUTS orgTimeouts; /* Original COM-Port Time-out  */
unsigned int nakDelay;    /* Delay before DATA_NAK will be send */
unsigned int UART_verbose = 0; /* verbose mode flag, default off */

unsigned char initializeCommPort(unsigned char* comPort, long rate, unsigned char parity);
unsigned long int calcTimeout(unsigned long startTime);
void ReadGargbageUntilTimeout(void);
void delay(unsigned int time);
void clearBuffers();


/*******************************************************************************
*Function:    setVerbose
*Description: sets Verbose mode to on or off for debugging
*Parameters: 
*             int val   1 == verbose mode on
                        0 == verbose mode off
*Returns:
*             none
*******************************************************************************/
void UART_setVerbose(unsigned int val)
{
	UART_verbose = val;
}

/*******************************************************************************
*Function:    UART_GetVerbose
*Description: returns UART_verbose
*Parameters:
*             none
*Returns:
*             unsigned int         returns value of UART_verbose
*******************************************************************************/
unsigned int UART_GetVerbose()
{
	return UART_verbose;
}

/*******************************************************************************
*Function:    initialize_BSL_5438
*Description: Initializes the COM port and invokes the BSL.
*Parameters: 
*             char* comPort           a string for the COM port ie "COM1"
*             unsigned int  mode      determines which mode will be entered
*Returns:
*             none
*******************************************************************************/
unsigned char UART_initialize_BSL_5438(unsigned char* comPort, long rate, unsigned int mode, unsigned char family)
{
	initializeCommPort(comPort, rate, 0);
	invokeBSL(mode);
	return BSL_ACK;
}
/*******************************************************************************
*Function:    initialize_BSL
*Description: Initializes the COM port and invokes the BSL.
*Parameters: 
*             char*         comPort   a string for the COM port ie "COM1"
*             unsigned int  mode      determines which mode will be entered
*Returns:
*             none
*******************************************************************************/
unsigned char UART_initialize_BSL(unsigned char* comPort, long rate, unsigned int mode, unsigned char family)
{
	unsigned char success = initializeCommPort(comPort, rate, 1);
	invokeBSL(mode);
	if ((family == FAMILY_MSP432))
	{
		changeCommBaudRate(rate);
		if ((mode == UART) && (rate >= 9600) && (rate <= 115200))
		{
			short int i = 0;
			unsigned char reply;
			dataBuffer sync;
			sync.data[0] = BAUD_SYNC;
			sync.size = 1;
			do
			{
				writeByte(sync.data, sync.size);
				i++;
				delay(10);
				reply = readByte();
			} while ((reply != BSL_ACK) && (reply != BSL_WRONG_HEADER) && (i < 10));

			if (i < 10)
			{
				return BSL_ACK;
			}
			else
			{
				return reply;
			}
		}
		else if ((mode == I2C) && (rate >= 100000) && (rate <= 400001))
		{
			return BSL_ACK;
		}
		else if ((mode == SPI) && (rate >= 125000) && (rate <= 1000001))
		{
			return BSL_ACK;
		}
		else
		{
			return BSL_NACK;
		}
	}
	else if (success == 0)
	{
		return BSL_ACK;
	}
	else
	{
		return BSL_NACK;
	}
}

/*******************************************************************************
*Function:    invokeBSL
*Description: toggles the TEST and RESET pins in order to invoke the BSL
*Parameters: 
*             unsigned int mode      determines which mode will be entered
*Returns:
*             none
*******************************************************************************/
void invokeBSL(unsigned int mode)
{
	SetRSTpin(0); 
	SetTESTpin(0);
	delay(250);

	SetRSTpin(0);  /* RST  pin: GND */
	SetTESTpin(0); /* TEST pin: GND */   delay(10);
	SetTESTpin(1); /* TEST pin: Vcc */   delay(10);
	SetTESTpin(0); /* TEST pin: GND */   delay(10);
	SetTESTpin(1); /* TEST pin: Vcc */   delay(10);
	SetRSTpin(1);  /* RST  pin: Vcc */   delay(10);
	SetTESTpin(0); /* TEST pin: GND */   delay(10);

	delay(350);
	PurgeComm(hComPort, PURGE_TXCLEAR);
	PurgeComm(hComPort, PURGE_RXCLEAR);
	
	if (mode == I2C)
	{
		changeCommBaudRate(100001);
		delay(100);
		changeCommBaudRate(100000);
	}
	else if (mode == SPI)
	{
		changeCommBaudRate(125001);
		delay(100);
		changeCommBaudRate(125000);
	}
	else if (mode == UART)
	{
		changeCommBaudRate(9601);
		delay(100);
		changeCommBaudRate(9600);
	}
	delay(350);
}

/*******************************************************************************
*Function:    SetRSTpin
*Description: Controls RST Pin
*Parameters: 
*             BOOL level            0: GND; 1: VCC
*Returns:
*             none
*******************************************************************************/
void SetRSTpin(BOOL level)
{
	comDCB.fDtrControl = level ? DTR_CONTROL_ENABLE : DTR_CONTROL_DISABLE;
	SetCommState(hComPort, &comDCB);
}

/*******************************************************************************
*Function:    SetTESTpin
*Description: Opens a TXT file for writing with append
*Parameters: 
*             BOOL level            0: GND; 1: VCC
*Returns:
*             none
*******************************************************************************/
void SetTESTpin(BOOL level)
{
	comDCB.fRtsControl = level ? RTS_CONTROL_DISABLE : RTS_CONTROL_ENABLE ;
	SetCommState(hComPort, &comDCB);
}

/*******************************************************************************
*Function:    writeByte
*Description: writes bytes to the COM port
*Parameters: 
*             unsigned char * byte             The byte to write to the COM port
*             unsigned int    size             Indicates the number of bytes that are transmitted
*Returns:
*             none
*******************************************************************************/
void writeByte(unsigned char * byte, unsigned int size)
{
	unsigned int dwWrite;
	PurgeComm(hComPort, PURGE_RXCLEAR | PURGE_RXABORT);
	WriteFile(hComPort, byte, size, &dwWrite, NULL);
}

/*******************************************************************************
*Function:    readByte
*Description: reads a single byte from the COM port buffer
*Parameters: 
*             none
*Returns:
*             unsigned char       The read byte
*******************************************************************************/
unsigned char readByte()
{
	unsigned char byte = TIMEOUT_ERROR;
	unsigned int dwRead;
	unsigned int errors;
	unsigned int error_flag = 0;
	unsigned int timeout = GetTickCount()+ TIMEOUT_MS;
	do
	{
		ClearCommError(hComPort, &errors, &comState);
		error_flag = GetTickCount() > timeout;
	}
	while ((comState.cbInQue == 0) && (error_flag == 0));

	if (error_flag == 0)
	{
		ReadFile(hComPort, &byte, 1, &dwRead, NULL);
	}
	 if (UART_verbose)
	{
		printf("{%2.2x} ", (byte&0xFF));
	}
	return byte;
}

/*******************************************************************************
*Function:    changeCommBaudRate
*Description: changes the baud rate for communication
*Parameters: 
*             unsigned int rate      The rate to change as defined in the header file
*Returns:
*             none
*******************************************************************************/
void changeCommBaudRate(unsigned int rate)
{
	GetCommState(hComPort, &comDCB);
	comDCB.BaudRate = rate;
	SetCommState(hComPort, &comDCB);
}

/*******************************************************************************
*Function:    initializeCommPort
*Description: initializes a COM port for communication
*Parameters: 
*             unsigned char *comPort    A string for the com port ie. "COM1"
*             unsigned char  parity     even or no parity
*Returns:
*             none
*******************************************************************************/
unsigned char initializeCommPort(unsigned char* comPort, long rate, unsigned char parity)
{
	COMMTIMEOUTS timeouts;
	unsigned int dwCommEvents;
	hComPort = CreateFile(comPort, GENERIC_READ | GENERIC_WRITE, 0, 0, OPEN_EXISTING, 0, 0);
	SetupComm(hComPort, QUEUE_SIZE, QUEUE_SIZE); 

	GetCommTimeouts(hComPort, &orgTimeouts);
	/* Set Windows timeout values (disable build-in timeouts): */
	timeouts.ReadIntervalTimeout         = 0;
	timeouts.ReadTotalTimeoutMultiplier  = 0;
	timeouts.ReadTotalTimeoutConstant    = 0;
	timeouts.WriteTotalTimeoutMultiplier = 0;
	timeouts.WriteTotalTimeoutConstant   = 0;

	SetCommTimeouts(hComPort, &timeouts);

	dwCommEvents= EV_RXCHAR | EV_TXEMPTY | EV_RXFLAG | EV_ERR;
	SetCommMask(hComPort, dwCommEvents);

	GetCommState(hComPort, &comDCB);

	comDCB.BaudRate = rate;
	comDCB.ByteSize = 8;
	nakDelay = (DWORD)((11 * MAX_UART_FRAME_SIZE) / (float)(rate / 1000));

	if (parity)
	{
		comDCB.Parity  = EVENPARITY;
	}
	else
	{
		comDCB.Parity  = NOPARITY;
	}
	comDCB.StopBits    = ONESTOPBIT;
	comDCB.fBinary     = TRUE; /* Enable Binary Transmission */
	comDCB.fParity     = TRUE; /* Enable Parity Check        */
	comDCB.ErrorChar   = (char)0xEF; 
	/* Char. w/ Parity-Err are replaced with 0xff 
	*(if fErrorChar is set to TRUE) 
	*/
	comDCB.fRtsControl = RTS_CONTROL_ENABLE; /* For power supply */
	comDCB.fDtrControl = DTR_CONTROL_ENABLE; /* For power supply */
	comDCB.fOutxCtsFlow= FALSE;
	comDCB.fOutxDsrFlow= FALSE;
	comDCB.fOutX       = FALSE;
	comDCB.fInX        = FALSE;
	comDCB.fNull       = FALSE;

	comDCB.fErrorChar  = FALSE;

	/* Assign new state: */
	SetCommState(hComPort, &comDCB);

	/* Clear buffers: */
	clearBuffers();
	if (hComPort != INVALID_HANDLE_VALUE)
	{
		return BSL_ACK;
	}
	else
	{
		return BSL_NACK;
	}
}

/*******************************************************************************
*Function:    clearBuffers
*Description: purges the Buffers
*Parameters: 
*             none
*Returns:
*             none
*******************************************************************************/
void clearBuffers(void)
{
	PurgeComm(hComPort, PURGE_TXCLEAR | PURGE_TXABORT);
	PurgeComm(hComPort, PURGE_RXCLEAR | PURGE_RXABORT);
}

/*******************************************************************************
*Function:    ReadGargbageUntilTimeout
*Description: if a problem with RX from the 430 happens, this function reads from the 430
*             until a timeout occurs, so communication can begin again correctly
*Parameters: 
*             none
*Returns:
*             none
*******************************************************************************/
void ReadGargbageUntilTimeout(void)
{
	char rxed_byte = 0;
#ifdef DEBUG_ERRORS
	printf("\n---------------ERROR on Packet RX---------------\n");
	printf("header:\t%2.2x\n", (RX_header&0xFF));
	printf("size:\t%x\n", (RX_size&0xFF));
	printf("packets:\t");
#endif
	do
	{
		rxed_byte = readByte();
#ifdef DEBUG_ERRORS
		printf ("(%2.2x)", (rxed_byte&0xFF));
#endif
	} while (rxed_byte!= (char)TIMEOUT_ERROR);
#ifdef DEBUG_ERRORS
	printf("\nchecksum low:\t%2.2x\n", (RX_checksum_low&0xFF));
	printf("checksum high:\t%2.2x\n", (RX_checksum_high&0xFF));
	printf("------------------------------------------------\n");
#endif
}

/*******************************************************************************
*Function:    delay
*Description: delays program execution by a certain number of miliseconds
*Parameters: 
*             DWORD time            Number of miliseconds to pause
*Returns:
*             none
*******************************************************************************/
void delay(unsigned int time) /* exported! */
{ 
#ifndef WIN32
	unsigned long int startTime= GetTickCount();
	while (calcTimeout(startTime) < time);
#else
	Sleep(time);
#endif
}

/*******************************************************************************
*Function:    calcTimeout
*Description: Calculates the difference between startTime and the acutal windows time
*Parameters: 
*             unsigned long startTime       The start time
*Returns:
*             unsigned long int             The difference between start Time and current time
*******************************************************************************/
unsigned long int calcTimeout(unsigned long startTime)
{
	return((unsigned long int)(GetTickCount() - startTime));
}